import requests

import vdcpy.config as config
import vdcpy.site as vdc_site


def get_projects(
    project_id=None, title=None, depositor_email=None, size=None,
    visibility=None):
    """
    Get a list of projects from the VDC API.
    """
    # Get a list of sites.
    site_list = vdc_site.get_sites()
    project_list_all_sites = []
    for site_info in site_list:
        
        query = f'http://{site_info["host"]}:5000/{config.PROJECT_NAMESPACE}'
        if project_id is not None:
            query += f'?id={project_id}'
        if title is not None:
            if project_id is None:
                query += f'?title={title}'
            else:
                query += f'&title={title}'
        if depositor_email is not None:
            if project_id is None and title is None:
                query += f'?depositor_email={depositor_email}'
            else:
                query += f'&depositor_email={depositor_email}'
        if size is not None:
            if project_id is None and title is None and depositor_email is None:
                query += f'?size={size}'
            else:
                query += f'&size={size}'
        if visibility is not None:
            if project_id is None and title is None and depositor_email is None \
                and size is None:
                query += f'?visibility={visibility}'
            else:
                query += f'&visibility={visibility}'
        try:
            response = requests.get(query)
            if response.status_code == 200:
                project_list = response.json()
                for project_info in project_list:
                    project_info['host'] = site_info['name']
                project_list_all_sites += project_list
            else:
                 print(
                    f'Error getting projects from {site_info["name"]} ' + \
                    f'({site_info["host"]}).' + \
                    f' Status code: {response.status_code}'
                )
        except requests.exceptions.ConnectionError:
            print(
                f'Error connecting to {site_info["name"]} ' + \
                f'({site_info["host"]}).'
            )
    return project_list_all_sites


def add_project(project_info, auth_token=None):
    """
    Add a project to the VDC API.
    """
    query = f'{config.API_URL}/{config.PROJECT_NAMESPACE}'
    if auth_token is not None:
        headers = {'Authorization': auth_token}
    else:
        headers = None
    response = requests.post(query, json=project_info, headers=headers)
    if response.status_code == 201:
        return response.json()
    else:
        raise Exception(
            f'Error adding project: {response.status_code} - {response.json()}'
        )


def update_project(project_id, project_info, auth_token=None):
    """
    Update a project in the VDC API.
    """
    query = f'{config.API_URL}/{config.PROJECT_NAMESPACE}/{project_id}'
    if auth_token is not None:
        headers = {'Authorization': auth_token}
    else:
        headers = None
    response = requests.put(query, json=project_info, headers=headers)
    if response.status_code == 200:
        return response.json()
    else:
        raise Exception(f'Error updating project: {response.status_code}')


def delete_project(project_id, auth_token=None):
    """
    Delete a project from the VDC API.
    """
    query = f'{config.API_URL}/{config.PROJECT_NAMESPACE}/{project_id}'
    if auth_token is not None:
        headers = {'Authorization': auth_token}
    else:
        headers = None
    response = requests.delete(query, headers=headers)
    if response.status_code == 204:
        return True
    else:
        raise Exception(f'Error deleting project: {response.status_code}')
